/*
	Liberty Unleashed Speedo Script
	--------------------------------------------------
	A speedometer script for Liberty Unleashed.
	Press 'L' to toggle vehicle lights.
	Use '/speedo <on/off>' to enable/disable the speedo.
	
	Author: Juppi
	Version: 1.0.1
*/

// Speedo settings
g_bEnabled <- true;
g_vSpeedoPos <- VectorScreen( ScreenWidth - 280, ScreenHeight - 325 );

// Warning light status
g_iLastLightUpdate <- 0;
g_WarningLightState <- 0;
g_WarningLightToggle <- false;

// Cache stuff
g_pLocalPlayer <- FindLocalPlayer();
g_pLocalVehicle <- null;
g_bLocalDriving <- false;
g_bInVehicle <- false;


/*
 *	LU Events
 */
 
 function onScriptLoad()
{
	print( "Liberty Unleashed Speedo Script" );
	print( "-------------------------------" );
	print( "Version 1.0.1, written by Juppi" );
	print( "Press 'L' to toggle vehicle lights on/off" );
	print( "Use '/speedo <on/off>' to enable/disable the speedo" );
	
	BindKey( 'L', BINDTYPE_DOWN, "ToggleVehicleLights" );
	
	// Create sprites
	::g_SpeedoDisk <- GUISprite( "disc.png", ::g_vSpeedoPos );
	::g_SpeedoDisk.Visible = false;
	::g_SpeedoNeedle <- GUISprite( "needle.png", ::g_vSpeedoPos, 0.0, VectorScreen( 128, 128 ) );
	::g_SpeedoNeedle.Visible = false;
	
	local vLight1Pos = VectorScreen( ::g_vSpeedoPos.x + 70, ::g_vSpeedoPos.y + 240 );
	local vLight2Pos = VectorScreen( ::g_vSpeedoPos.x + 100, ::g_vSpeedoPos.y + 240 );
	local vLight3Pos = VectorScreen( ::g_vSpeedoPos.x + 130, ::g_vSpeedoPos.y + 240 );
	local vLight4Pos = VectorScreen( ::g_vSpeedoPos.x + 160, ::g_vSpeedoPos.y + 240 );
	
	::g_Warning_on <- GUISprite( "warning_on.png", vLight1Pos ); ::g_Warning_on.Visible = false;
	::g_Warning_on2 <- GUISprite( "warning_on2.png", vLight1Pos ); ::g_Warning_on2.Visible = false;
	::g_Warning_off <- GUISprite( "warning_off.png", vLight1Pos ); ::g_Warning_off.Visible = false;
	::g_Temp_on <- GUISprite( "temp_on.png", vLight2Pos ); ::g_Temp_on.Visible = false;
	::g_Temp_off <- GUISprite( "temp_off.png", vLight2Pos ); ::g_Temp_off.Visible = false;
	::g_Handbrake_on <- GUISprite( "handbr_on.png", vLight3Pos ); ::g_Handbrake_on.Visible = false;
	::g_Handbrake_off <- GUISprite( "handbr_off.png", vLight3Pos ); ::g_Handbrake_off.Visible = false;
	::g_Lights_on <- GUISprite( "lights_on.png", vLight4Pos ); ::g_Lights_on.Visible = false;
	::g_Lights_off <- GUISprite( "lights_off.png", vLight4Pos ); ::g_Lights_off.Visible = false;
	
	local vGearPos = VectorScreen( ::g_vSpeedoPos.x + 120, ::g_vSpeedoPos.y + 198 );
	
	::g_Gear1 <- GUISprite( "gear1.png", vGearPos ); ::g_Gear1.Visible = false;
	::g_Gear2 <- GUISprite( "gear2.png", vGearPos ); ::g_Gear2.Visible = false;
	::g_Gear3 <- GUISprite( "gear3.png", vGearPos ); ::g_Gear3.Visible = false;
	::g_Gear4 <- GUISprite( "gear4.png", vGearPos ); ::g_Gear4.Visible = false;
	::g_Gear5 <- GUISprite( "gear5.png", vGearPos ); ::g_Gear5.Visible = false;
	::g_GearR <- GUISprite( "gearr.png", vGearPos ); ::g_GearR.Visible = false;
	
	// Add layers for each sprite so that they are rendered
	AddGUILayer( ::g_SpeedoDisk );
	AddGUILayer( ::g_SpeedoNeedle );
	AddGUILayer( ::g_Warning_on );
	AddGUILayer( ::g_Warning_on2 );
	AddGUILayer( ::g_Warning_off );
	AddGUILayer( ::g_Temp_on );
	AddGUILayer( ::g_Temp_off );
	AddGUILayer( ::g_Handbrake_on );
	AddGUILayer( ::g_Handbrake_off );
	AddGUILayer( ::g_Lights_on );
	AddGUILayer( ::g_Lights_off );
	AddGUILayer( ::g_Gear1 );
	AddGUILayer( ::g_Gear2 );
	AddGUILayer( ::g_Gear3 );
	AddGUILayer( ::g_Gear4 );
	AddGUILayer( ::g_Gear5 );
	AddGUILayer( ::g_GearR );
	
	return 1;
}
 
function onClientEnteredVehicle( veh, seat )
{
	// Check if the local player is driving the vehicle, cache the result for future use
	if ( !seat ) ::g_bLocalDriving = true;
	else ::g_bLocalDriving = false;
	
	// Store some info for our rendering hook
	::g_bInVehicle = true;
	::g_pLocalVehicle = veh;
	
	if ( ::g_bEnabled ) ToggleSpeedo( true );
	
	return 1;
}

function onClientExitedVehicle( veh )
{
	// Reset data
	::g_bInVehicle = false;
	::g_pLocalVehicle = null;
	::g_bLocalDriving = false;
	
	ToggleSpeedo( false );
	
	return 1;
}

function onClientRender()
{
	// Only process speedo if the local player is in a vehicle! (And they have activated the speedo)
	if ( ::g_bEnabled && ::g_bInVehicle )
	{
		local pVehicle = ::g_pLocalVehicle;
		
		local fSpeed = GetSpeedFromVelocity( pVehicle.Velocity );
		local fHealth = pVehicle.Health - 250;
		local fCarHealth = fHealth < 0 ? 0.0 : fHealth;
		
		::g_SpeedoNeedle.Rotation = -( fSpeed * 230 ); // Speed in KPH: 1 kph = 1 degree
		
		// If the vehicle is a car display its current gear
		if ( IsVehicleCar( pVehicle.Model ) ) DisplayVehicleGear( pVehicle.Gear );
		// Otherwise just show 1 (This is to fix the 'reverse' bug on dodo/boats etc, they only have gear 0)
		else DisplayVehicleGear( 1 );
		
		// Warning light processing (only based on vehicle engine health atm) - Only process if the local player is driving
		if ( g_bLocalDriving )
		{
			if ( fHealth < 200.0 )
			{
				local iTimestamp = GetTickCount();
				if ( iTimestamp - ::g_iLastLightUpdate > 150 )
				{
					if ( ::g_WarningLightToggle )
					{
						::g_Warning_on.Visible = false;
						::g_Warning_on2.Visible = true;
						::g_Warning_off.Visible = false;
					}
					else
					{
						::g_Warning_on.Visible = false;
						::g_Warning_on2.Visible = false;
						::g_Warning_off.Visible = true;
					}
					
					::g_Temp_on.Visible = true;
					::g_Temp_off.Visible = false;
					
					::g_WarningLightToggle = !g_WarningLightToggle;
					::g_WarningLightState = 1;
					
					::g_iLastLightUpdate = iTimestamp;
				}
			}
			
			else if ( fHealth < 350.0 )
			{
				::g_Warning_on.Visible = false;
				::g_Warning_on2.Visible = true;
				::g_Warning_off.Visible = false;
				
				::g_Temp_on.Visible = true;
				::g_Temp_off.Visible = false;
				
				::g_WarningLightState = 1;
			}
			else if ( fHealth < 500.0 )
			{
				::g_Warning_on.Visible = true;
				::g_Warning_on2.Visible = false;
				::g_Warning_off.Visible = false;
				
				::g_Temp_on.Visible = false;
				::g_Temp_off.Visible = true;
				
				::g_WarningLightState = 1;
			}
			else if ( ::g_WarningLightState != 0 )
			{
				::g_Warning_on.Visible = false;
				::g_Warning_on2.Visible = false;
				::g_Warning_off.Visible = true;
				
				::g_Temp_on.Visible = false;
				::g_Temp_off.Visible = true;
				
				::g_WarningLightState = 0;
			}
		}
	}
	
	return 1;
}

function onClientKeyStateChange( oldkeys, newkeys )
{
	if ( ::g_bEnabled )
	{
		if ( ::g_bInVehicle )
		{
			if ( ::g_bLocalDriving )
			{
				if ( ( ( newkeys & KEY_INCAR_HANDBRAKE ) == KEY_INCAR_HANDBRAKE ) && ( ( oldkeys & KEY_INCAR_HANDBRAKE ) != KEY_INCAR_HANDBRAKE ) )
				{
					// Handbrake used. Show the light!
					::g_Handbrake_on.Visible = true;
					::g_Handbrake_off.Visible = false;
				}
				else if ( ( ( newkeys & KEY_INCAR_HANDBRAKE ) != KEY_INCAR_HANDBRAKE ) && ( ( oldkeys & KEY_INCAR_HANDBRAKE ) == KEY_INCAR_HANDBRAKE ) )
				{
					// Handbrake released. Kill the light!
					::g_Handbrake_on.Visible = false;
					::g_Handbrake_off.Visible = true;
				}
			}
		}
	}
	
	return 1;
}

function onClientCommand( cmd, text )
{
	if ( cmd == "speedo" )
	{
		if ( ( text ) && ( text == "off" ) )
		{
			::g_bEnabled = false;
			if ( ::g_bInVehicle ) ToggleSpeedo( false );
		}
		else
		{
			::g_bEnabled = true;
			if ( ::g_bInVehicle ) ToggleSpeedo( true );
		}
	}
	
	return 1;
}


/*
 *	Misc Functions
 */

function ToggleSpeedo( enabled )
{
	if ( enabled )
	{
		// When the speedo is enabled, disable GTA HUD stuff except for the radar
		SetHUDItemEnabled( HUD_ARMOUR, false );
		SetHUDItemEnabled( HUD_CLOCK, false );
		SetHUDItemEnabled( HUD_HEALTH, false );
		SetHUDItemEnabled( HUD_MONEY, false );
		SetHUDItemEnabled( HUD_WANTED, false );
		SetHUDItemEnabled( HUD_WEAPON, false );
		
		// ...and display our own stuff instead
		::g_SpeedoDisk.Visible = true;
		::g_SpeedoNeedle.Visible = true;
		::g_Gear1.Visible = true;
		
		// Some stuff (warning lights) should only be applied for the driver
		if ( ::g_bLocalDriving )
		{
			::g_Warning_off.Visible = true;
			::g_Temp_off.Visible = true;
			::g_Handbrake_off.Visible = true;
			
			if ( ::g_pLocalVehicle.LightState == LIGHTSTATE_ON ) ::g_Lights_on.Visible = true;
			else ::g_Lights_off.Visible = true;
		}
	}
	
	else
	{
		// Re-enable GTA HUD stuff
		SetHUDItemEnabled( HUD_ARMOUR, true );
		SetHUDItemEnabled( HUD_CLOCK, true );
		SetHUDItemEnabled( HUD_HEALTH, true );
		SetHUDItemEnabled( HUD_MONEY, true );
		SetHUDItemEnabled( HUD_WANTED, true );
		SetHUDItemEnabled( HUD_WEAPON, true );
		
		// Hide all our stuff!
		::g_SpeedoDisk.Visible = false;
		::g_SpeedoNeedle.Visible = false;
		
		::g_Warning_off.Visible = false;
		::g_Warning_on.Visible = false;
		::g_Warning_on2.Visible = false;
		::g_Temp_off.Visible = false;
		::g_Temp_on.Visible = false;
		::g_Handbrake_off.Visible = false;
		::g_Handbrake_on.Visible = false;
		::g_Lights_off.Visible = false;
		::g_Lights_on.Visible = false;
		
		::g_Gear1.Visible = false;
		::g_Gear2.Visible = false;
		::g_Gear3.Visible = false;
		::g_Gear4.Visible = false;
		::g_Gear5.Visible = false;
		::g_GearR.Visible = false;
	}
}

function IsVehicleCar( model )
{
	switch ( model )
	{
		case VEH_PREDATOR:
		case VEH_TRAIN:
		case VEH_CHOPPER:
		case VEH_DODO:
		case VEH_RCBANDIT:
		case VEH_AIRTRAIN:
		case VEH_DEADDODO:
		case VEH_SPEEDER:
		case VEH_REEFER:
			return false;
	}
	
	return true;
}

function GetSpeedFromVelocity( v )
{
	return sqrt( v.x*v.x + v.y*v.y + v.z*v.z );
}

function DisplayVehicleGear( gear )
{
	switch ( gear )
	{
		case 0:
			::g_Gear1.Visible = false;
			::g_Gear2.Visible = false;
			::g_Gear3.Visible = false;
			::g_Gear4.Visible = false;
			::g_Gear5.Visible = false;
			::g_GearR.Visible = true;
			break;
		
		case 1:
			::g_Gear1.Visible = true;
			::g_Gear2.Visible = false;
			::g_Gear3.Visible = false;
			::g_Gear4.Visible = false;
			::g_Gear5.Visible = false;
			::g_GearR.Visible = false;
			break;
		
		case 2:
			::g_Gear1.Visible = false;
			::g_Gear2.Visible = true;
			::g_Gear3.Visible = false;
			::g_Gear4.Visible = false;
			::g_Gear5.Visible = false;
			::g_GearR.Visible = false;
			break;
		
		case 3:
			::g_Gear1.Visible = false;
			::g_Gear2.Visible = false;
			::g_Gear3.Visible = true;
			::g_Gear4.Visible = false;
			::g_Gear5.Visible = false;
			::g_GearR.Visible = false;
			break;
		
		case 4:
			::g_Gear1.Visible = false;
			::g_Gear2.Visible = false;
			::g_Gear3.Visible = false;
			::g_Gear4.Visible = true;
			::g_Gear5.Visible = false;
			::g_GearR.Visible = false;
			break;
		
		case 5:
			::g_Gear1.Visible = false;
			::g_Gear2.Visible = false;
			::g_Gear3.Visible = false;
			::g_Gear4.Visible = false;
			::g_Gear5.Visible = true;
			::g_GearR.Visible = false;
			break;
	}
}

function ToggleVehicleLights()
{
	if ( ::g_bInVehicle )
	{
		// Check that the player is actually driving the vehicle
		if ( ::g_bLocalDriving )
		{
			local bLights = ::g_pLocalVehicle.LightState == LIGHTSTATE_ON ? false : true;
			CallServerFunc( "speedo/speedo_server.nut", "ToggleVehicleLights", ::g_pLocalVehicle, bLights ); // We do this to sync the light state for everyone
			
			if ( ::g_bEnabled )
			{
				if ( bLights )
				{
					::g_Lights_on.Visible = true;
					::g_Lights_off.Visible = false;
				}
				else
				{
					::g_Lights_on.Visible = false;
					::g_Lights_off.Visible = true;
				}
			}
		}
	}
}
